<div class="uk-clearfix">
<div class="uk-card uk-card-body"><a class="uk-link-heading" href="docs-year-%s.html" uk-toggle=""><span class="uk-heading-large uk-text-bolder uk-text-danger">%s</span></a></div>


