= About this theme

++++
<div class="uk-flex uk-flex-center"><h1 class="uk-heading-large">%s</h1></div>
<h1 class="uk-heading-small">Description</h1> <!-- Name -->
<div class="uk-text-lead uk-text-primary">%s</div>
<h1 class="uk-heading-small">Version</h1> <!-- Version -->
<div class="uk-text-lead uk-text-primary">%s</div>
++++