        <tr>
            <td>
                <div class="uk-text-bolder uk-text-secondary uk-text-bold uk-text-meta">%s</div>
            </td>
            <td><div class="uk-flex uk-flex-row uk-flex-wrap uk-text-meta uk-padding-small uk-padding-remove-top uk-padding-remove-bottom">%s</div></td>
        </tr>

