= Newspaper

// END-OF-HEADER. DO NOT MODIFY OR DELETE THIS LINE

++++
<ul class="uk-flex-center" uk-tab>
    <li><a href="#">Today</a></li>
    <li><a href="#">Yesterday</a></li>
    <li><a href="#">This week</a></li>
    <li><a href="#">This month</a></li>
    <li><a href="#">Past month</a></li>
</ul>

<ul class="uk-switcher uk-margin">
    <li>
        <div class="uk-grid-match uk-child-width-1-3@s uk-flex-left" uk-grid>
%s
        </div>
    </li>
    <li>
        <div class="uk-grid-match uk-child-width-1-3@s uk-flex-left" uk-grid>
%s
        </div>
    </li>
    <li>
        <div class="uk-grid-match uk-child-width-1-3@s uk-flex-left" uk-grid>
%s
        </div>
    </li>
    <li>
        <div class="uk-grid-match uk-child-width-1-3@s uk-flex-left" uk-grid>
%s
        </div>
    </li>
    <li>
        <div class="uk-grid-match uk-child-width-1-3@s uk-flex-left" uk-grid>
%s
        </div>
    </li>
</ul>
++++
