= About KB4IT

// END-OF-HEADER. DO NOT MODIFY OR DELETE THIS LINE


++++
<div class="uk-flex uk-flex-center"><h1 class="uk-heading-large">KB4IT</h1></div>
++++

=== Description

_KB4IT_ is a static website generator based on Asciidoctor sources mainly for technical documentation purposes.

This website was build using KB4IT v%s

For more information, visit my https://github.com/t00m/KB4IT[repository] at GitHub.

=== Copyright

Copyright (C) 2019-2020 mailto:tomasvirseda@gmail.com[Tomás Vírseda]

=== License

The code is licensed under the terms of the https://www.gnu.org/licenses/gpl-3.0.en.html[`GPL v3`] so you're free to *grab*, *extend*, *improve* and *fork the code* as you want.

