        <!-- DOC_CARD.tpl :: START -->
        <article class="uk-article">
            <div class="uk-grid-small uk-flex-middle" uk-grid>
                <div class="uk-width-expand">
                    <h1 class="uk-article-title">%s</h1>
                    <p class="uk-text-meta uk-margin-remove-top"><time datetime="%s">%s</time></p>
                </div>
            </div>
            <div class="uk-grid-small uk-child-width-auto" uk-grid>
                <div class="uk-flex uk-flex-right">
                    <ul class="uk-breadcrumb">
                        <li><span class="uk-text-meta uk-text-bold">Archived on&nbsp;</span>%s</li>
                    </ul>
                </div>
            </div>
        </article>
        <!-- DOC_CARD.tpl :: END -->
