<div>
    <div class="uk-card uk-card-hover uk-padding-small uk-padding-remove-bottom">
        <div class="uk-card uk-card-header uk-padding-remove-top uk-padding-remove-bottom uk-padding-remove-right">
            <div class="uk-grid-small uk-flex-middle" uk-grid>
                <div class="uk-width-auto uk-vertical-align">
                    <img class="uk-border uk-position-top-left" width="40" height="40"  src="%s">
                </div>
                <div class="uk-padding uk-padding-remove-top uk-padding-remove-right">
                    <h3 class="uk-text-meta uk-text-bold uk-text-justify"><a href="%s">%s</a></h3>
                    <p class="uk-text-meta uk-margin-remove-top uk-align-right">%s</p>
                </div>
            </div>
        </div>
    </div>
</div>
