                            <!-- DOC_CARD_FOOTER.tpl :: START -->
                            <div class="uk-card uk-padding-small uk-padding-remove-bottom">
                                <div class="uk-flex uk-flex-right">
                                    <ul class="uk-breadcrumb">
                                        <li uk-tooltip="Category">%s</li>
                                        <li uk-tooltip="Scope">%s</li>
                                    </ul>
                                </div>
                            </div>
                            <!-- DOC_CARD_FOOTER.tpl :: END -->
                        <!-- DOC_CARD.tpl :: END -->