= %s

++++
    <!-- KEY_PAGE.tpl -->
    <!-- Tabs header :: START-->
    <ul class="uk-flex-center" uk-tab>
        <li><a href="#">Cloud</a></li>
        <li><a href="#">Stats</a></li>
    </ul>
    <!-- Tabs header :: END -->

    <!-- Tabs content :: START-->
    <ul class="uk-switcher uk-margin">
        <li> <!-- Cloud :: Start -->
%s
        </li> <!-- Cloud :: End -->

        <li> <!-- Stats :: Start -->
%s
        </li> <!-- Stats :: End-->
    </ul>
    <!-- Tabs content :: END-->
++++
