= Stats

++++
    <!-- INDEX_TAB_STATS.tpl :: START -->
    <div class="uk-grid-small" uk-grid>
        <div class="uk-width-expand uk-text-bold" uk-leader>
            <a class="uk-link-heading" href="all.html">Documents</a>
        </div>
        <div>
            <a href="all.html">%d</a>
        </div> <!-- Num of documents -->
    </div>
    <div class="uk-grid-small" uk-grid>
        <div class="uk-width-expand uk-text-bold" uk-leader>
            <a class="uk-link-heading" href="properties.html">Properties</a>
        </div>
        <div><a href="properties.html">%d</a></div> <!-- Num of properties -->
    </div>
    <!-- Property leader items -->
%s
    <!-- Property leader items -->
++++