        <tr>
            <td>
                <a class="uk-link-text" href="%s.html">
                    <div class="uk-text-bolder uk-text-secondary uk-text-bold uk-text-meta">%s</div>
                </a>
            </td>
            <td><div class="uk-flex uk-flex-row uk-flex-wrap uk-text-meta">%s</div></td>
        </tr>

