            <div class="uk-navbar-center">
                <!-- DOCUMENT TITLE :: START -->
<span class="uk-text-danger uk-text-truncate">%s</span>
                <!-- DOCUMENT TITLE :: END -->
            </div>
            <div class="uk-navbar-right noprint">
                <ul class="uk-navbar-nav">
                    <li>
                        <a class="uk-button uk-card uk-card-hover uk-link-heading" href="#">Help</a>
                        <div class="uk-navbar-dropdown">
                            <ul class="uk-nav uk-navbar-dropdown-nav">
                                <li class="uk-link-toggle">
                                    <a class="uk-button uk-card uk-card-hover uk-link-heading" href="about.html">About</a>
                                </li>
                            </ul>
                        </div>
                    </li>
                </ul>
            </div>
        </nav>
    </div>
</div>
<!-- CONTENT :: START -->
<div class="uk-container uk-container-center">
