<li>
        <div class="uk-flex uk-flex-center"><span class="uk-text-meta">%d/%d documents</span></div>
        <div class="uk-child-width-1-3@m uk-grid-collapse" uk-grid>
    %s
        </div>
</li>
