    <div class="uk-grid-small" uk-grid>
        <div class="uk-width-1-6@m"></div>
        <div class="uk-width-expand uk-text-bold" uk-leader><a class="uk-link-heading" href="%s.html">%s</a></div>
        <div><a href="%s.html">%d</a></div>
    </div>
