<div>
    <div class="uk-card uk-card-default uk-card-hover">
        <div class="uk-card-header">
            <div class="uk-grid-small uk-flex-middle" uk-grid>
                <div class="uk-width-auto">
                    <img class="uk-border" width="40" height="40"  src="%s">
                </div>
                <div class="uk-width-expand">
                    <h3 class="uk-card-title uk-margin-remove-bottom uk-text-lead"><a href="%s">%s</a></h3>
                    <p class="uk-text-meta uk-margin-remove-top">%s</p>
                </div>
            </div>
        </div>
        <div class="uk-card-body">
            <p class="uk-text-bold">%s</p>
        </div>
    </div>
</div>
