= Properties

++++
<div class="uk-card uk-border-rounded uk-card-body uk-auto@m">
    <div class="uk-card-header">
        <div class="uk-grid-medium uk-flex-middle uk-child-width-expand" uk-grid>
            <div class="uk-width-expand">
                <div class="uk-flex uk-flex-center uk-flex-wrap">
%s
                </div>
            </div>
        </div>
    </div>
</div>
++++