            <table class="uk-table uk-table-small">
              <tr>
                <td><a class="uk-link-toggle" href="Author.html"><div class="uk-text-bold uk-text-secondary">Author</div></a></td>
                <td><div class="uk-flex uk-flex-row uk-flex-wrap">%s</div></td>
                <td><a class="uk-link-toggle" href="Category.html"><div class="uk-text-bold uk-text-secondary">Category</div></a></td>
                <td><div class="uk-flex uk-flex-row uk-flex-wrap">%s</div></td>
                <td><a class="uk-link-toggle" href="Scope.html"><div class="uk-text-bold uk-text-secondary">Scope</div></a></td>
                <td><div class="uk-flex uk-flex-row uk-flex-wrap">%s</div></td>
              </tr>
            </table>
