<!DOCTYPE Html>
<html lang="en">
<head>
    <title>KB4IT - %s</title>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="generator" content="Asciidoctor 2.0.10">
    <meta name="description" content="KB4IT document">
    <meta name="author" content="KB4IT by t00mlabs.net">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="stylesheet" href="resources/common/uikit/css/uikit.min.css">
    <link rel="stylesheet" href="resources/common/uikit/css/coderay-asciidoctor.css">
    <link rel="stylesheet" href="resources/common/uikit/css/print.css" type="text/css" media="print" />
    <link rel="stylesheet" href="resources/common/uikit/css/kb4it.css">
    <script src="resources/common/uikit/js/uikit.min.js"></script>
    <script src="resources/common/uikit/js/uikit-icons.min.js"></script>
</head>
<body>
<div>
<div id="container-1" class="uk-container uk-container-center">
    <div id="kb4it-menu" style="z-index: 980;" uk-sticky="show-on-up: true">
        <nav class="uk-navbar-container uk-border-rounded uk-card-hover uk-margin" style="background-color: white;" uk-navbar>
            <div class="uk-navbar-left noprint">
                <ul class="uk-navbar-nav">
                    <li class="uk-link-toggle">
                        <a class="uk-logo uk-card uk-card-hover" href="index.html">
                            <img src="resources/themes/%s/images/logo.png" alt="">
                        </a>
                    </li>
                    <li>
                        <a class="uk-button uk-card uk-card-hover uk-button uk-card uk-card-hover uk-link-heading" href="#">Go To</a>
                        <div class="uk-navbar-dropdown">
                            <ul class="uk-nav uk-navbar-dropdown-nav">
                                <li class="uk-link-toggle">
                                    <a class="uk-button uk-card uk-card-hover uk-link-heading" href="newspapers.html">Newspapers</a>
                                </li>
                                <li class="uk-link-toggle">
                                    <a class="uk-button uk-card uk-card-hover uk-link-heading" href="today.html">Today</a>
                                </li>
                                <li class="uk-link-toggle">
                                    <a class="uk-button uk-card uk-card-hover uk-link-heading" href="yesterday.html">Yesterday</a>
                                </li>
                                <li class="uk-link-toggle">
                                    <a class="uk-button uk-card uk-card-hover uk-link-heading" href="week.html">This week</a>
                                </li>
                                <li class="uk-link-toggle">
                                    <a class="uk-button uk-card uk-card-hover uk-link-heading" href="month.html">This month</a>
                                </li>
                                <li class="uk-link-toggle">
                                    <a class="uk-button uk-card uk-card-hover uk-link-heading" href="past.html">Past month</a>
                                </li>
                                <li class="uk-link-toggle">
                                    <a class="uk-button uk-card uk-card-hover uk-link-heading" href="Tag.html">Tag cloud</a>
                                </li>
                            </ul>
                        </div>
                    </li>
                </ul>
                <ul class="uk-navbar-nav">
                    <!-- MENU CONTENTS :: START -->
%s
                    <!-- MENU CONTENTS :: END -->
                </ul>
            </div>
