                            <li class="noprint">
                                <a class="uk-button uk-card uk-card-hover uk-link-heading" href="#">Contents</a>
                                <div class="uk-navbar-dropdown">
                                    <ul class="uk-nav uk-navbar-dropdown-nav">
%s
                                    </ul>
                                </div>
                            </li>